#include "blk_dev.h"
#include "ata.h"
#include "../mem/alloc.h"
#include "../util/util.h"

#define ATA_SECTOR_SIZE	(512)

struct blk_dev *blk_devs[4]; 
unsigned int blk_devs_cnt;

struct kmem_cache *mem_ata_drives;

static int ata_blk_dev_read(struct blk_dev *dev, unsigned char *buf, uint32_t blk_offset, uint32_t blk_cnt);
static int ata_blk_dev_write(struct blk_dev *dev, unsigned char *buf, uint32_t blk_offset, uint32_t blk_cnt);
static void ata_blk_dev_ctor(void *o, struct kmem_cache *cache);
static int ata_blk_dev_test_and_create(int bus, int slave);

/**
 * Vyhleda a inicializuje blokova zarizeni
 */
int blk_dev_detect()
{
	mem_ata_drives = kmem_cache_create(sizeof(struct ata_blk_dev), NULL, ata_blk_dev_ctor, NULL);
	ata_blk_dev_test_and_create(ATA_PRIMARY_BUS, 0);
	ata_blk_dev_test_and_create(ATA_PRIMARY_BUS, 1);
	ata_blk_dev_test_and_create(ATA_SECONDARY_BUS, 0);
	ata_blk_dev_test_and_create(ATA_SECONDARY_BUS, 1);
	return 0;
}

static void ata_blk_dev_ctor(void *o, struct kmem_cache *cache)
{
	struct ata_blk_dev *dev = o;
	dev->blk_dev.block_size = ATA_SECTOR_SIZE;
	dev->blk_dev.read       = ata_blk_dev_read;
	dev->blk_dev.write      = ata_blk_dev_write;
}

static int ata_blk_dev_read(struct blk_dev *dev, unsigned char *buf, uint32_t blk_offset, uint32_t blk_cnt)
{
	struct dev_ata *dev_ata = &((struct ata_blk_dev *) dev)->dev_ata;
	while (blk_cnt > 0) {
		/* ata_read umi prenest jen 255 sektoru v jednom bloku */
		int count = blk_cnt <= 255 ? blk_cnt : 255;
		int status = ata_read(dev_ata, blk_offset, count, buf); 
		if (status) return status;

		blk_cnt    -= count;
		buf        += ATA_SECTOR_SIZE * count;
		blk_offset += count;
	}
	return EOK;
}

static int ata_blk_dev_write(struct blk_dev *dev, unsigned char *buf, uint32_t blk_offset, uint32_t blk_cnt)
{
	return EOK;
}

/**
 * Vytvori nove ATA zarizeni a zaradi jej na seznam blokovych zarizeni
 */
static int ata_blk_dev_test_and_create(int bus, int slave)
{
	kprint("drive ");
	kprint((bus == ATA_PRIMARY_BUS) ? "primary " : "secondary ");
	kprint(slave ? "slave: " : "master: ");

	if (ata_detect(bus, slave) == ATA_PATA) {
		struct ata_blk_dev *dev = kmem_cache_alloc(mem_ata_drives);
		struct dev_ata *dev_ata = &(dev->dev_ata);

		ata_init(dev_ata, bus, slave);
		dev->blk_dev.block_cnt = ata_drive_size(dev_ata);
		kprint_i(dev->blk_dev.block_cnt);
		kprint(" blocks\r\n");
		blk_devs[blk_devs_cnt++] = (struct blk_dev *) dev;
		return EOK;
	}
	kprint("not found\r\n");
	return EATA_DEV_FAIL;
}


