#ifndef IRQ_H
#define IRQ_H

#include <stdint.h>

#define PIC1_CMD   (0x20)           /* port ridiciho registru PIC1 */
#define PIC1_DATA  (0x21)           /* datovy port PIC1 */
#define PIC2_CMD   (0xa0)           /* port ridiciho registr PIC2 */
#define PIC2_DATA  (0xa1)           /* datovy port PIC2 */

#define PIC_MASTER      (PIC1_DATA)
#define PIC_SLAVE       (PIC2_DATA)

struct idt_entry {
	uint16_t offset_1;  // offset (0..15)
	uint16_t selector;  // selector kodoveho segmentu z obsluznou rutinou
	uint8_t  zero;      // rezervovano -> 0
	uint8_t  type_attr; // typ preruseni
	uint16_t offset_2;  // offset (16..31)
};

void irq_init();
void irq_load_idt(unsigned char entries_cnt, struct idt_entry *entries); 
void irq_set_handler(unsigned char irq, void *handler);
void irq_set_syscall_handler(unsigned char irq, void *handler);

void pic_init();
void pic_enable_irq(unsigned pic, uint8_t mask);
void pic_disable_irq(unsigned pic, uint8_t mask);

/* obalujici rutiny implementovane v assembleru */
void isr_0();
void isr_1();
void isr_keyb();
void isr_syscall();

#endif
