/*
 * Vstupni bod jadra operacniho systemu.
 *
 * Tento zdrojovy kod by mel mit prave jednu funkci, ktera je zavolana
 * po prepnuti do 32bitoveho rezimu. (Pokud by zdrojovy soubor obsahoval
 * dalsi funkce, muze je prekladac preskladat a dostaneme nezadouci chovani.)
 */


#include "irq/irq.h"

#include "mem/alloc.h"
#include "mem/pg_alloc.h"

#include "dev/term.h"
#include "dev/keyb.h"
#include "dev/blk_dev.h"

#include "util/io.h"
#include "util/util.h"
#include "util/string.h"

#include "fs/mufs.h"
#include "tasks/readelf.h"
#include "tasks/tasks.h"

#define KERNEL_MEM_PAGES	(256)
#define USER_CODE_START		(0x00200000)

struct pg_zone kernel_mem_zone;
struct page kernel_mem_pages[KERNEL_MEM_PAGES];

struct kmem_cache mem_ata_drives;
struct mufs root_fs;

extern struct term term;
extern struct blk_dev *blk_devs[];
extern unsigned int blk_devs_cnt;
extern struct task active_task;

unsigned char buf[4096];

void load_tss();

void kernel_main()
{
	load_tss();
	term_init();
	kprint_color(3, 0);
	pg_zone_init(&kernel_mem_zone, KERNEL_MEM_PAGES, kernel_mem_pages, (unsigned char *) 0x00100000);
	kmem_init();
	keyb_init();
	irq_init();
	kprint_color(2, 0);
	kprint("starting...\r\n");

	blk_dev_detect();
	if (blk_devs_cnt > 1) {
		kprint("initilizing file system: ");

		int status = mufs_open(&root_fs, blk_devs[1]);
		if (status) {
			kprint("failed\r\n");
		} else {
			kprint("success\r\n");
		}
// varianta 1
//		struct mufs_fd *f = mufs_file_open(&root_fs, "foo.txt", 0);
//		mufs_file_read(f, buf, 200);
//		kprint(buf);
//		kprint("\r\n");
//		mufs_file_close(f);
//		
//		while (1);

// varianta 2
//		Elf32_Ehdr ehdr;
//		Elf32_Phdr phdr;
//		//struct mufs_fd *f = mufs_file_open(&root_fs, "hello01", 0);
//		struct mufs_fd *f = mufs_file_open(&root_fs, "hello02", 0);
//		status = readelf_file_header(f, &ehdr);
//		if (status == EOK) {
//			for (int i = 0; i < ehdr.e_phnum; i++) {
//				readelf_program_header(f, i, &ehdr, &phdr);
//				mufs_file_seek(f, phdr.p_offset, SEEK_SET);
//
//				if (phdr.p_type != PT_LOAD) continue;
//
//				if (phdr.p_vaddr < USER_CODE_START) {
//					kprint("Attempt to corrupt kernel memory\r\n");
//					while (1);
//				}
//
//				memset((void *) phdr.p_vaddr, 0x00, phdr.p_memsz);
//				mufs_file_read(f, (unsigned char *) phdr.p_vaddr, phdr.p_filesz);
//			}
//
//			/* primy skok v ring0 */
//			void (*start_fun)() = (void (*)()) ehdr.e_entry;
//			start_fun();
//			
//			
//			/* prepnuti do ring3 */
//			//jumpto_task(ehdr.e_entry);
//			while (1);
//
//		} else {
//			kprint("Invalid file\r\n");
//		}
//		mufs_file_close(f);


// varianta 3
		jumpto_scheduler();	
	}
	
	while (1) {
	}
}

