#ifndef ALLOC_H
#define ALLOC_H
#include <stdint.h>
#include "../util/list.h"

#define PAGE_SIZE         (4096)

/**
 * struktura popisujici jeden slab (deska/plat)
 */
struct slab {
	struct slab *next;  /* odkazy na dalsi/predchozi slaby */
	struct slab *prev;
	struct page *page;  /* stranka s danym slabem */
	uint16_t obj_cnt;   /* pocet aktivnich objektu v danem slabu */
	uint16_t bmap_size; /* velikost bitmapy v bytech */
};

/**
 * Struktura popisujici alokator pro jeden typ objektu
 */
struct kmem_cache {
	unsigned int obj_size;      /* velikost objektu zaokrouhlena na slova */
	unsigned int objs_in_slab;  /* pocet objektu v jednom slabu */
	unsigned int bmap_size;     /* velikost bitmapy volnych bloku */
	void *private;              /* pomocna hodnota, kterou lze specifikovat dalsi vlasnosti slabu */

	void (*ctor)(void *, struct kmem_cache *); /* konstruktor */
	void (*dtor)(void *, struct kmem_cache *); /* destruktor */

	struct slab *partial_slabs; /* seznam castecne zaplnenych slabu */
	struct slab *full_slabs;    /* seznam zcela zaplnenych slabu */
	struct slab *free_slabs;    /* seznam volnych slabu */
};

/**
 * Vytvori alokator pro objekty daneho typu
 */
struct kmem_cache *kmem_cache_create(unsigned int size,
	void *private,
	void (*ctor)(void *, struct kmem_cache *),
	void (*dtor)(void *, struct kmem_cache *));

/**
 * Inicializuje slab alokator pro alokaci dalsich alokatoru
 */
void kmem_init();

/**
 * Alokuje ze slabu jeden objekt
 */
void *kmem_cache_alloc(struct kmem_cache *cache);

/**
 * Uvolni objekt v danem slabu
 */
void kmem_cache_free(struct kmem_cache *cache, void *obj);

/**
 * Uvolni volne slaby (postara se o volani destruktoru)
 */
void kmem_cache_shrink(struct kmem_cache *cache);
#endif
