#ifndef PG_ALLOC_H
#define PG_ALLOC_H

#include <stdint.h>

#define MAX_ORDER       (16)  /* celkove nejvetsi rad stranek */

/**
 * Struktura popisujici seznam volnych stranek,
 * je ulozena ve volne strance.
 */
struct free_page {
	struct free_page *prev;
	struct free_page *next;
	struct page *page;
};

/** struktura popisujici stranku, resp. souvisly blok stranek */
struct page
{
	unsigned char *addr; /* zacatek stranky v pameti */
	int index;           /* index stranky, pro ladeni a snazsi manipulaci, lze vypustit */
	uint8_t order;       /* pocet stranek (2^n), ktere tato struktura reprezentuje */
	uint8_t free;        /* priznak, zda je stranka volna */
};

/** struktura popisujici buddy alokator */
struct pg_zone
{
	unsigned char *base;                    /* adresa, kde jsou alokovane stranky */
	struct free_page *free_list[MAX_ORDER]; /* pro kazdy rad si udrzuje seznam volnych bloku */
	struct page *pages;                     /* pole s jednotlivymi metadaty k jednotlivym strankam */
	uint8_t max_order;                      /* maximalni rad pouzitelny v dane zone */
};

/**
 * Alokuje stranku dane velikosti, size musi byt mocninou 2.
 */
struct page *pg_alloc(struct pg_zone *zone, uint32_t size);

/**
 * Uvolni danou stranku a spoji ji s jejim buddym,
 * pokud existuje. 
 */
void pg_free(struct pg_zone *zone, struct page *page);

/**
 * Inicializuje zonu se strankami.
 * page_count -- pocet stranek (musi byt mocnina dvou)
 * page -- pole struktur popisujici jednotlive stranky
 * base -- adresa, kde jsou stranky ulozeny
 */
void pg_zone_init(struct pg_zone *zone, int page_count, struct page *pages, unsigned char *base);

#endif
