[ORG 0x7c00]
[BITS 16] 
    mov ax, 0
    mov ss, ax              ; nastavime vrchol zasobniku
    mov sp, 0x7c00          ; segment i offset
 
    mov si, mode16_txt
    call print

;
; nacteni kodu jadra
;
    mov bx, kernel_blocks    ; pocet sektoru jadra, co je potreba nahrat
load_kernel:
    mov  cx, bx              ; pocet sektoru, ktere se budou nacitat
    cmp  cx, max_blocks_at_once
    jb   skip_size_adjustment
    mov  cx, max_blocks_at_once
skip_size_adjustment:
    mov  [blkcnt], cx
    call read_disk
    mov  cx, [blkcnt]        ; pocet skutecne nactenych sektoru
    sub  bx, cx
    jz   load_kernel_complete
    add  [src], cx   
    shl  cx, 5               ; posune segment o pocet sektoru * 512
    add  [dest + 2], cx
    jmp  load_kernel

load_kernel_complete:




;
; prechod do chraneneho rezimu
;
 
    cli                     ; zablokujeme preruseni
    lgdt [gdt_desc]         ; nacteme do GDTR deskriptor GDT
    
    mov eax, cr0            ; v ridicim registru CR0 nastavime
    or eax, 1               ; bit indikujici chraneny mod
    mov cr0, eax          
 
    jmp 0x08:main32         ; skok do 32bitoveho segmentu (selektor 0x08)
 
    mov si, mode16_fail
    call print
  
stop:
    jmp stop               ; vstoupime do nekonecne smycky 

;
; pomocna funkce pro vypis na terminal
; adresa retezce je predana v si
;
print:
    mov ah, 0x0e           ; identifikator vystupni operace

print_char:
    mov al, [si]           ; precteme jeden znak
    cmp al, 0              ; pokud je \0,
    jz print_done          ; skoncime

    int 0x10               ; vypiseme znak
    add si, 1              ; a pokracujeme dalsim znakem
    jmp print_char
print_done:
    ret

;
; pomocna funkce pro cteni z disku
;
read_disk:
    mov si, dapack         ; adresa paketu s adresami pro prenos dat 
    mov ah, 0x42           ; identifikator operace
    mov dl, 0x80           ; cislo disku 0 (plus 0x80)
    int 0x13
    jc disk_err
    ret

disk_err:
    mov si, disk_fail
    call print
    jmp stop



;
; 32bitovy kod
;

[BITS 32]
main32:
    mov ax, 0x10
    mov ds, ax
    mov ss, ax
    mov es, ax
    mov fs, ax
    mov gs, ax

    mov esp, 0x70000
    mov esi, mode32_txt
    mov edi, 0xb8000

;
; vypis realizovany jeste v ramci assembleru
;

print_char32:
    mov al, [esi]
    cmp al, 0
    je kernel_exec

    mov [edi], al
    mov byte [edi + 1], 0x1f

    add esi, 1
    add edi, 2
    jmp print_char32

;
; zavolani vstupniho bodu jadra (funkce kernel_main)
;

kernel_exec:
    call 0x10000

stop32:
   jmp stop32

;
; GDT
;

gdt:
gdt_null:            ; povinny null-zaznam v GDT
    dd 0x0
    dd 0x0

gdt_cs:              ; descriptor kodoveho segmentu
    dw 0xffff        ; limit (bity 0-15)
    dw 0x0000        ; base (bity 0-15)
    db 0x00          ; base (bity 16-23)
    db 10011010b     ; 1: present; 00: ring0; 1: not-system-seg.; 1: code-seg.; 0: not-conforming; 1: read/write; 0: accessed
    db 11001111b     ; 1: granularity = pages; 1: 32bit mode; 00: reserved; 1111: limit (bity 16-19)
    db 0x00          ; base (bity 24-31)

gdt_ds:              ; descriptor datoveho segmentu
    dw 0xffff        ; limit (bity 0-15)
    dw 0x0000        ; base (bity 0-15)
    db 0x00          ; base (bity 16-23)
    db 10010010b     ; 1: present; 00: ring0; 1: not-system-seg.; 0: data-seg.; 0: not-conforming; 1: read/write; 0: accessed
    db 11001111b     ; 1: granularity = pages; 1: 32bit mode; 00: reserved; 1111: limit (bity 16-19)
    db 0x00          ; base (bity 24-31)

tss:
    dw 0x00ff        ; limit (bity 0-15) 255
    dw 0x0000        ; base (bity 0-15)
    db 0x08          ; base (bity 16-23)
    db 10001001b     ; 1: present; 00: ring0; system-seg.; 01001: task state segment (not busy)
    db 00000000b     ; 0: granularity = bytes; 000: reserved; 0000: limit (bity 16-19)
    db 0x00          ; base (bity 24-31)

gdt_cs3:              ; descriptor kodoveho segmentu
    dw 0xffff        ; limit (bity 0-15)
    dw 0x0000        ; base (bity 0-15)
    db 0x00          ; base (bity 16-23)
    db 11111010b     ; 1: present; 11: ring0; 1: not-system-seg.; 1: code-seg.; 0: not-conforming; 1: read/write; 0: accessed
    db 11001111b     ; 1: granularity = pages; 1: 32bit mode; 00: reserved; 1111: limit (bity 16-19)
    db 0x00          ; base (bity 24-31)

gdt_ds3:              ; descriptor datoveho segmentu
    dw 0xffff        ; limit (bity 0-15)
    dw 0x0000        ; base (bity 0-15)
    db 0x00          ; base (bity 16-23)
    db 11110010b     ; 1: present; 11: ring3; 1: not-system-seg.; 0: data-seg.; 0: not-conforming; 1: read/write; 0: accessed
    db 11001111b     ; 1: granularity = pages; 1: 32bit mode; 00: reserved; 1111: limit (bity 16-19)
    db 0x00          ; base (bity 24-31)


gdt_end:

gdt_desc:            ; descriptor tabulky GDT
    dw gdt_end - gdt - 1      ; velikost tabulky - 1
    dd gdt

;
; struktura obsahujici paket s adresami (disk address packet)
; urcena pro cteni dat z disku
;
dapack:
    db 0x10		; velikost prikazu
    db 0		; vzdycky 0
blkcnt:	
    dw 0                ; pocet bloku k precteni/nactenych (hodnota zmenena po provedeni preruseni) 
dest:                   ; adresa ciloveho bufferu (0x0001:0x0000)	
     dw 0x0000		; offset v ramci segmentu (0x0000)
     dw 0x1000		; segment (0x0001)
src:	
    dd 1		; adresa bloku na disku
    dd 0		; 



;
; podpurne konstanty
;

kernel_blocks equ (KERNEL_SIZE + 511) / 512 ; pocet sektoru s jadrem
max_blocks_at_once equ 127 ; pocet sektoru, ktere se maji nacist najednou

mode16_txt:
    db '16bit mode', 13, 10, 0

mode32_txt:
    db '*** 32bit mode ***', 0

mode16_fail:
    db 'switch failure', 13, 10, 0

disk_fail:
    db 'disk failure', 13, 10, 0

    times 510 -($-$$) db 0 ; vyplnime zbyvajici prostor 0
    dw 0xaa55              ; signatura MBR
