#ifndef READELF_H
#define READELF_H

#include <stdint.h>

#define EOK           (0)
#define EIO_ERR       (1)
#define EINVALID_FILE (2)

typedef uint32_t Elf32_Addr;
typedef uint16_t Elf32_Half;
typedef int32_t  Elf32_Off;
typedef int32_t  Elf32_Sword;
typedef uint32_t Elf32_Word;

/* hodnoty e_type */
#define ET_NONE   (0)      /* No file type */
#define ET_REL    (1)      /* Relocatable file */
#define ET_EXEC   (2)      /* Executable file */
#define ET_DYN    (3)      /* Shared object file */
#define ET_CORE   (4)      /* Core file */
#define ET_LOPROC (0xff00) /* Processor-specific */
#define ET_HIPROC (0xffff) /* Processor-specific */

/* hodnoty e_machine */
#define EM_NONE  (0) /* No machine */
#define EM_M32   (1) /* AT&T WE 32100 */
#define EM_SPARC (2) /* SPARC */
#define EM_386   (3) /* Intel 80386 */
#define EM_68K   (4) /* Motorola 68000 */
#define EM_88K   (5) /* Motorola 88000 */
#define EM_860   (7) /* Intel 80860 */
#define EM_MIPS  (8) /* MIPS RS3000 */

/* hodnoty e_version */
#define EV_NONE    (0) /* Invalid version */
#define EV_CURRENT (1) /* Current version */

/* hodnoty e_ident */
#define EI_MAG0    (0) /* File identification */
#define EI_MAG1    (1) /* File identification */
#define EI_MAG2    (2) /* File identification */
#define EI_MAG3    (3) /* File identification */
#define EI_CLASS   (4) /* File class */
#define EI_DATA    (5) /* Data encoding */
#define EI_VERSION (6) /* File version */
#define EI_PAD     (7) /* Start of padding bytes */
#define EI_NIDENT (16)

/* magicke cislou souboru */
#define ELFMAG0 (0x7f) /* e_ident[EI_MAG0] */
#define ELFMAG1 ('E')  /* e_ident[EI_MAG1] */
#define ELFMAG2 ('L')  /* e_ident[EI_MAG2] */
#define ELFMAG3 ('F')  /* e_ident[EI_MAG3] */

/* hodnota EI_CLASS */
#define ELFCLASSNONE (0) /* Invalid class */
#define ELFCLASS32 (1)   /* 32-bit objects */
#define ELFCLASS64 (2)   /* 64-bit objects */

/* hodnota EI_DATA */
#define ELFDATANONE (0) /* Invalid data encoding */
#define ELFDATA2LSB (1) /* little endian */
#define ELFDATA2MSB (2) /* big endian */


#define PT_LOAD	(1)

/** hlavicka celeho objektoveho souboru */
typedef struct {
	unsigned char e_ident[EI_NIDENT]; /* identifikator souboru ELF */
	Elf32_Half e_type;        /* typ souboru, viz ET_* */
	Elf32_Half e_machine;     /* architektura, viz EM_* */
	Elf32_Word e_version;     /* verze formatu, viz EV_* */
	Elf32_Addr e_entry;       /* logicka adresa vstupni funkce */
	Elf32_Off  e_phoff;       /* adresa hlavicek programu */
	Elf32_Off  e_shoff;       /* adresa hlavicek sekci */
	Elf32_Word e_flags;       /* priznaky */
	Elf32_Half e_ehsize;      /* velikost hlavicky souboru */
	Elf32_Half e_phentsize;   /* velikost hlavicky programu */
	Elf32_Half e_phnum;       /* pocet hlavicek programu */
	Elf32_Half e_shentsize;   /* velikost hlavicky sekci */
	Elf32_Half e_shnum;       /* pocet hlavicek sekci */
	Elf32_Half e_shstrndx;    /* index tabulky s retezci */
} Elf32_Ehdr;


typedef struct {
	Elf32_Word p_type;        /* typ zaznamu */
	Elf32_Off  p_offset;      /* offset v ramci souboru */
	Elf32_Addr p_vaddr;       /* logicka adresa */
	Elf32_Addr p_paddr;       /* fyzicka adresa */
	Elf32_Word p_filesz;      /* velikost dat v souboru */
	Elf32_Word p_memsz;       /* velikost dat v pameti, p_memsz >= p_filesz */
	Elf32_Word p_flags;       /* priznaky */
	Elf32_Word p_align;       /* zarovnani */
} Elf32_Phdr;


/** Nacte hlavicku objektoveho souboru. */
int readelf_file_header(struct mufs_fd *f, Elf32_Ehdr *ehdr);

/** Nacte "program header" s danym indexem */
int readelf_program_header(struct mufs_fd *f, int index, Elf32_Ehdr *ehdr, Elf32_Phdr *phdr);
#endif
