global load_tss
global jumpto_task
global jumpto_scheduler
extern task_scheduler

TSS_LOCATION equ 0x80000    ;; umisteni TSS  
KERNEL_STACK equ 0x50000    ;; zacatek jaderneho zasobniku
USER_STACK   equ 0x003f0000 ;; zacatek zasobniku v uzivatelskem prostoru

GDT_CS0      equ 0x08       ;; selector kodoveho segmentu jadra
GDT_DS0      equ 0x10       ;; selector dataveho segmentu jadra
GDT_TSS      equ 0x18       ;; selector TSS
GDT_CS3	     equ (0x20 | 3) ;; selector kodoveho segmentu uzivatelskeho prostoru
GDT_DS3	     equ (0x28 | 3) ;; selector kodoveho segmentu uzivatelskeho prostoru

;;
;; provede nastaveni registru vcetne zasobniku pro spusteni noveho procesu
;; a preda rizeni planovaci uloh
;;
jumpto_scheduler:
        mov ax, GDT_DS0
	mov ds, ax
        mov es, ax
        mov fs, ax
        mov gs, ax

	mov esp, KERNEL_STACK
	pushf
	or dword [esp], 0x200 ; povoli preruseni

	push dword GDT_CS0
	push task_scheduler
	iret

;; inicializuje TSS na nejnutnejsi minimum
;; nahraje selector TSS do registru TR
load_tss:
	mov eax, TSS_LOCATION
	mov dword [eax + 4],   KERNEL_STACK
	mov word  [eax + 8],   GDT_DS0
	mov word  [eax + 102], 104 ; offset IOPB

	mov ax, GDT_TSS
	ltr ax
	ret

;;
;; provede prepnuti do ring3, vcetne nastaveni zasobniku
;; a zacne provadet kod od adresy ENTRY_POINT
;;
jumpto_task:
        mov ecx, [esp + 4]   ;; entry point
        mov ax, GDT_DS3
	mov ds, ax
        mov es, ax
        mov fs, ax
        mov gs, ax

        mov eax, USER_STACK
        push GDT_DS3
	push eax             ;; puvodni ESP (tj. stack uzivatelskeho prostoru)

	pushf

        push GDT_CS3
	push ecx             ;; entry point
	iretd


